@@ -1,7 +1,7 @@
 /*
-Copyright(c) 2013-2014 Ilya Kreymer. Released under the GNU General Public License.
+Copyright(c) 2013-2018 Rhizome and Ilya Kreymer. Released under the GNU General Public License.
 
-This file is part of pywb, https://github.com/ikreymer/pywb
+This file is part of pywb, https://github.com/webrecorder/pywb
 
     pywb is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@ if (window.location.hash) {
     if (_pywbvid == "html" || _pywbvid == "flash") {
         var YT_W_E_RX = /^(https?:\/\/.*youtube.com)\/(watch|embed).*$/;
 
-        if (wbinfo.url.match(YT_W_E_RX)) {
+        if (window.wbinfo && window.wbinfo.url.match(YT_W_E_RX)) {
             // special case: prevent yt player from being inited
             Object.defineProperty(window, 'yt', {writeable: false});
             Object.defineProperty(window, 'ytplayer', {writeable: false});
@@ -45,12 +45,21 @@ if (window.location.hash) {
 
 __wbvidrw = (function() {
 
-    var found_embeds = false;
+    if (!window.wbinfo) {
+        return;
+    }
+
+    var checked_embeds = false;
 
     var FLASH_PLAYER = wbinfo.static_prefix + "/flowplayer/flowplayer-3.2.18.swf";
 
+    var replay_prefix = wbinfo.prefix;
+    if (wbinfo.mod) {
+        replay_prefix += wbinfo.mod + "/";
+    }
+
     function check_videos() {
-        if (found_embeds) {
+        if (checked_embeds) {
             return;
         }
 
@@ -73,7 +82,7 @@ __wbvidrw = (function() {
         handle_all_embeds();
         handle_all_objects();
 
-        found_embeds = true;
+        checked_embeds = true;
 
         handle_yt_videos(_pywbvid);
 
@@ -82,26 +91,62 @@ __wbvidrw = (function() {
         //_wb_wombat.add_tag_handler("object", handle_all_objects);
     }
 
+    function check_broken_vimeo()
+    {
+        if (document.querySelector("video")) {
+            return;
+        }
+
+        if (document.querySelector("object") || document.querySelector("embed")) {
+            return;
+        }
+
+        var player = document.getElementById("player");
+
+        if (!player) {
+            return;
+        }
+
+        player.classList.remove("loading");
+
+        // Add placeholder embed
+        var embed = document.createElement("embed");
+        embed.src = wbinfo.url;
+        player.appendChild(embed);
+
+        if (!window.MutationObserver) {
+            handle_embed_tag(embed);
+        }
+    }
+
     function handle_embed_tag(elem)
     {
         var src = elem.getAttribute("src");
-        if (src) {
-            if (elem._vidrw) {
-                return false;
-            }
-
-            // already rewritten
-            if (elem.outerHTML.indexOf(FLASH_PLAYER) >= 0) {
-                return false;
-            }
-
-            elem._vidrw = true;
-
-            check_replacement(elem, src);
-            return true;
+        if (!src) {
+            return false;
+        }
+        
+        if (elem._vidrw) {
+            return false;
         }
 
-        return false;
+        // already rewritten
+        if (elem.outerHTML.indexOf(FLASH_PLAYER) >= 0) {
+            return false;
+        }
+
+        elem._vidrw = true;
+        
+        if (src.indexOf("ustream.tv/flash") >= 0) {
+            var flashvars = elem.getAttribute("flashvars");
+            var res = flashvars.match(/[vc]id=([\d]+)/);
+            if (res) {
+                src = "http://www.ustream.tv/recorded/" + res[1];
+            }
+        }
+
+        check_replacement(elem, src);
+        return true;
     }
 
     function handle_object_tag(elem)
@@ -124,10 +169,37 @@ __wbvidrw = (function() {
                 var name = child.getAttribute("name");
                 name = name.toLowerCase();
 
-                if (name == "movie") {
+                if (name == "movie" || name == "src") {
                     var value = child.getAttribute("value");
                     obj_url = value;
                 }
+
+                //flashvars extraction: work in progress: for now, only ustream
+                if (name == "flashvars") {
+                    if (wbinfo.url.indexOf("ustream") >= 0) {
+                        var value = child.getAttribute("value");
+
+                        var pairs = value.split('&');
+
+                        var result = {};
+                        pairs.forEach(function(pair) {
+                            if (obj_url) {
+                                return;
+                            }
+                            pair = pair.split('=');
+                            var qval = decodeURIComponent(pair[1] || '');
+                            //TODO: allow for picking from multiple urls
+                            if (qval.indexOf("http") == 0) {
+                                obj_url = qval;
+                                return;
+                            }
+                        });
+                    }
+
+                    //if (wbinfo.url.indexOf("livestream") >= 0) {
+                    obj_url = wbinfo.url;
+                    //}
+                }
             }
         }
 
@@ -185,6 +257,7 @@ __wbvidrw = (function() {
 
             //add placeholder child to remove
             var placeholder = document.createElement("div");
+            placeholder._no_rewrite = true;
             elem.appendChild(placeholder);
             check_replacement(placeholder, wbinfo.url);
         }
@@ -202,12 +275,19 @@ __wbvidrw = (function() {
                 if (_pywbvid == "html" || _pywbvid == "flash") {
                     do_yt_video_replace(player_div);
                 } else if (!wbinfo.is_live) {
-                    var player = window.yt.player.getPlayerByElement(player_div);
+                    var player = undefined;
+
+                    if (window.yt && window.yt.player && window.yt.player.getPlayerByElement) {
+                        player = window.yt.player.getPlayerByElement(player_div);
+                    }
 
                     if (player) {
                         _pywb_yt_err = function() {
-                            do_yt_video_replace(player_div);
+                            console.log("yt onError");
+                            console.log(arguments);
+                            //do_yt_video_replace(player_div);
                         }
+                        player.addEventListener("onStateChange", function() { console.log("yt stateChange"); });
 
                         player.addEventListener("onError", "_pywb_yt_err");
                     }
@@ -221,7 +301,7 @@ __wbvidrw = (function() {
                         var state = -1;
 
                         if (player && player.getPlayerState) {
-                           state = player.getPlayerState();
+                            state = player.getPlayerState();
                         }
 
                         // if no player or player is still buffering (is this ok), then replace
@@ -265,19 +345,46 @@ __wbvidrw = (function() {
 
         var xhr = new XMLHttpRequest();
         xhr._no_rewrite = true;
-        xhr.open('GET', wbinfo.prefix + 'vi_/' + src, true);
+
+        var info_url;
+
+        // if in proxy mode, access video info via special proxy magic path
+        // eg: http://pywb.proxy/<coll>/vi_/<url>
+        if (wbinfo.proxy_magic) {
+            info_url = window.location.protocol + "//" + wbinfo.proxy_magic + "/" + wbinfo.coll + "/";
+        } else {
+            info_url = wbinfo.prefix;
+        }
+
+        info_url += "vi_/" + src;
+
+        xhr.open('GET', info_url, true);
         xhr.onload = function() {
             if (xhr.status == 200) {
                 var videoinfo = JSON.parse(xhr.responseText);
 
-                // special case
+                // special cases
+                // Soundcloud
                 if (videoinfo._type == "playlist" && videoinfo.extractor == "soundcloud:playlist") {
                     var player_url = "https://w.soundcloud.com/player/?visual=true&show_artwork=true&url=" + encodeURIComponent(videoinfo.webpage_url);
                     do_replace_iframe(elem, player_url);
                     return;
                 }
 
-                // end special case
+                if (videoinfo._type == "playlist") {
+                    if (videoinfo.entries && videoinfo.entries.length > 0) {
+                        do_replace_video(elem, videoinfo.entries[0]);
+                        return;
+                    }
+                }
+
+                if (!videoinfo.formats && videoinfo.url && videoinfo.ext && videoinfo.format_id &&
+                    !(videoinfo.ext == "swf" && videoinfo.format_id == "0")) {
+                    videoinfo.formats = [{ext: videoinfo.ext,
+                                          url: videoinfo.url,
+                                          format_id: videoinfo.format_id
+                                         }];
+                }
 
                 if (videoinfo.formats) {
                     do_replace_video(elem, videoinfo);
@@ -320,6 +427,11 @@ __wbvidrw = (function() {
     function do_replace_elem(elem, replacement) {
         var tag_name = elem.tagName.toLowerCase();
 
+        if (!elem.parentNode) {
+            console.warn("vidrw: Missing Parent Node for Video Replace!");
+            return;
+        }
+
         if (tag_name == "iframe" || tag_name == "object") {
             elem.parentNode.replaceChild(replacement, elem);
         } else if (tag_name == "embed") {
@@ -330,7 +442,7 @@ __wbvidrw = (function() {
             elem.parentNode.replaceChild(replacement, elem);
 
         } else {
-           elem.parentNode.replaceChild(replacement, elem);
+            elem.parentNode.replaceChild(replacement, elem);
         }
     }
 
@@ -339,7 +451,7 @@ __wbvidrw = (function() {
         var thumb_url = null;
 
         if (info.thumbnail) {
-            thumb_url = wbinfo.prefix + info.thumbnail;
+            thumb_url = replay_prefix + info.thumbnail;
         }
 
         var tag_name = elem.tagName.toLowerCase();
@@ -356,6 +468,7 @@ __wbvidrw = (function() {
 
         // Try HTML5 Video
         var htmlelem = document.createElement("video");
+        htmlelem._no_rewrite = true;
 
         var replacement = null;
 
@@ -365,6 +478,7 @@ __wbvidrw = (function() {
             if (type) {
                 if (type == "audio") {
                     htmlelem = document.createElement("audio");
+                    htmlelem._no_rewrite = true;
                 }
                 if (_pywbvid != "flash") {
                     replacement = init_html_player(htmlelem, type, width, height, info, thumb_url);
@@ -375,7 +489,14 @@ __wbvidrw = (function() {
         var vidId = undefined;
 
         if (!replacement) {
+
+            // check format, if this is just an swf file, and not a video, nothing to replace!
+            if (get_format_ext(info) == "swf") {
+                return;
+            }
+
             replacement = document.createElement("div");
+            replacement._no_rewrite = true;
 
             vidId = "_wb_vid" + Date.now() + Math.random();
             replacement.setAttribute("id", vidId);
@@ -439,6 +560,7 @@ __wbvidrw = (function() {
         htmlelem.setAttribute("height", height);
         htmlelem.setAttribute("controls", "1");
         htmlelem.style.backgroundColor = "#000";
+        htmlelem.style.visibility = "visible";
 
         if (thumb_url) {
             htmlelem.setAttribute("poster", thumb_url);
@@ -454,6 +576,7 @@ __wbvidrw = (function() {
             }
 
             var replacement = document.createElement("div");
+            replacement._no_rewrite = true;
 
             var vidId = "_wb_vid" + Date.now();
             replacement.setAttribute("id", vidId);
@@ -465,11 +588,15 @@ __wbvidrw = (function() {
 
         for (var i = -1; i < info.formats.length; i++) {
             var source = document.createElement("source");
+            source._no_rewrite = true;
 
             var url, format;
 
             if (i < 0) {
                 url = info.url;
+                if (!url) {
+                    continue;
+                }
                 format = get_format_ext(info);
             } else {
                 if (!info.formats[i]._wb_canPlay) {
@@ -479,7 +606,7 @@ __wbvidrw = (function() {
                 format = get_format_ext(info.formats[i]);
             }
 
-            url = wbinfo.prefix + url;
+            url = replay_prefix + url;
             format = type + "/" + format;
 
             source.setAttribute("src", url);
@@ -502,7 +629,7 @@ __wbvidrw = (function() {
             height += "px";
         }
 
-        var style = 'width: ' + width + '; height: ' + height + '; display: block';
+        var style = 'width: ' + width + '; height: ' + height + '; display: block; visibility: visible !important';
 
         elem.style.cssText += ';' + style;
 
@@ -514,7 +641,7 @@ __wbvidrw = (function() {
             url = info.url;
         }
 
-        url = wbinfo.prefix + url;
+        url = replay_prefix + url;
 
         var config = {
             clip: {
@@ -532,7 +659,7 @@ __wbvidrw = (function() {
         };
 
         var do_embed = function() {
-            window.flashembed(div_id, opts, {"config": config});
+            var flash = window.flashembed(div_id, opts, {"config": config});
         };
 
         if (!window.flashembed) {
@@ -546,8 +673,64 @@ __wbvidrw = (function() {
         }
     }
 
-    document.addEventListener("DOMContentLoaded", function() {
-        window.setTimeout(check_videos, 200);
-    });
+    //============================================
+    function init_node_insert_obs(window)
+    {
+        function do_handle(te, func)
+        {
+            setTimeout(function() { func(te); }, 100);
+        }
+
+        var m = new MutationObserver(function(records, observer)
+                                     {
+            for (var i = 0; i < records.length; i++) {
+                var r = records[i];
+                if (r.type == "childList") {
+                    for (var j = 0; j < r.addedNodes.length; j++) {
+
+                        var elem = undefined;
+                        var tag_name = undefined;
+
+                        try {
+                            elem = r.addedNodes[j];
+                            tag_name = elem.tagName;
+                        } catch (e) {
+                            continue;
+                        }
+
+                        if (tag_name) {
+                            if (tag_name == "OBJECT") {
+                                do_handle(elem, handle_object_tag);
+                            } else if (tag_name == "EMBED") {
+                                do_handle(elem, handle_embed_tag);
+                            }
+                        }                           
+                    }
+                }
+            }
+        });
+
+        m.observe(window.document.documentElement, {
+            childList: true,
+            subtree: true,
+        });
+    }
+
+
+    if (!window.MutationObserver) {
+        document.addEventListener("DOMContentLoaded", function() {
+            window.setTimeout(check_videos, 200);
+        });
+    } else {
+        init_node_insert_obs(window);
+    }
+
+
+    // VIMEO FIX
+    if (wbinfo && wbinfo.url.indexOf("player.vimeo.com/") >= 0) {
+        document.addEventListener("DOMContentLoaded", function() {
+            window.setTimeout(check_broken_vimeo, 500);
+        });
+    }
 
 })();
